package benefactorkindboxreqhandler

import (
	"net/http"

	params "git.gocasts.ir/ebhomengo/niki/param"
	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/kind_box_req"
	"git.gocasts.ir/ebhomengo/niki/pkg/claim"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	echo "github.com/labstack/echo/v4"
)

// Add godoc
// @Summary      Add a new kind box request for a benefactor
// @Tags         Benefactors KindBoxReqs
// @Accept       json
// @Produce      json
// @Param        Request body   param.KindBoxReqAddRequest true "New kind box request details"
// @Success      200  {object}  param.KindBoxReqAddResponse
// @Failure      400  {string}  "Bad request"
// @Security     AuthBearerBenefactor
// @Router       /benefactors/kindboxreqs [post].
func (h Handler) Add(c echo.Context) error {
	req := param.KindBoxReqAddRequest{}
	if err := c.Bind(&req); err != nil {
		return c.JSON(http.StatusBadRequest, echo.Map{
			"message": errmsg.ErrBadRequest,
		})
		// TODO: return echo.NewHTTPError(http.StatusBadRequest, errmsg.ErrBadRequest) ؟؟؟
	}
	claims := claim.GetClaimsFromEchoContext(c)
	req.BenefactorID = claims.UserID

	resp, sErr := h.benefactorKindBoxReqSvc.Add(c.Request().Context(), req)
	if sErr != nil {
		msg, code := httpmsg.Error(sErr)
		if resp.FieldErrors != nil {
			return c.JSON(code, echo.Map{
				"message": msg,
				"errors":  resp.FieldErrors,
			})
		}

		return echo.NewHTTPError(code, msg)
	}

	go h.notificationSvc.KindBoxReqAdded(params.NotificationKindBoxReqAdded{
		KindBoxReqID: resp.KindBoxReq.ID,
	})

	return c.JSON(http.StatusCreated, resp)
}
